/*-
 * Copyright 2020 UPLEX Nils Goroll Systemoptimierung
 * All rights reserved.
 *
 * Author: Nils Goroll <nils.goroll@uplex.de>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include "config.h"

#include <dlfcn.h>
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

#include "vdef.h"
#include "vrt.h"

int
main(int argc, char **argv) {
	void *dlhdl;
	char buf[256];
	const struct vmod_data *d;
	const char *name;
	const char *file;

	if (argc < 3) {
		fprintf(stderr, "need 2 arguments\n");
		return (1);
	}

	name = argv[1];
	file = argv[2];


	dlhdl = dlopen(file, RTLD_LAZY | RTLD_LOCAL);
	if (dlhdl == NULL) {
		fprintf(stderr, "dlopen: %s\n", dlerror());
		return (1);
	}
	bprintf(buf, "Vmod_%s_Data", name);
	d = dlsym(dlhdl, buf);
	if (d == NULL) {
		fprintf(stderr, "dlsym: %s\n", dlerror());
		return (1);
	}

	printf("%s",d->proto);
	printf("\nstatic const struct vmod_data import_%s = {\n", buf);
	printf("\t.name =\t\t\"%s\",\n", d->name);
	printf("\t.file_id =\t\"%s\",\n", d->file_id);
	printf("\t.abi =\t\t\"%s\",\n", d->abi);
	printf("};\n");

	return (0);
}
